from PyChipsUser import *
import flash_lib as flash_lib
import flash_cmds as flash_cmds
import select_mcs as select_mcs
import readline
import time
import os
import sys

def print_menu():
    print "\nFlash actions:"
    print "Z                    zero flash"
    print "V                    read/verify flash"
    print "P                    program flash (includes zero and read/verify)"
    print "RF <addr> <no_pgs>   read flash <no_pgs> pages (default 1) from <addr>"
    print "RS <addr> <no_add>   read <no_add> addresses from spartan chip at <addr>"
    print "WS <addr> <value>    write <value> to <addr>"
    print "ST <verbosity>       Get status at some <verbosity> level between 0 and 2"
    print "BC <broadcast_cmd>   Send a broadcast command over the Fibers"
    print "Q                    quit\n"

def is_int(str):
  ent = str.upper()
  valid_hex = '0123456789ABCDEF'
  valid_num = '0123456789'
  if ent.startswith("0X"):
    for char in ent[2:]:
      if char not in valid_hex:
        return False
  else:
    for char in ent:
      if char not in valid_num:
        return False
  return True

def get_address_from_name(name="VERSION"):
    return flash_cmds.ttt.pyAddTbl.items[name]._address

def print_status(verbosity = 0):
    chA_errors = 0
    chB_errors = 0

    if verbosity >= 0:
        print __VERSION__
        print __SERIAL_NO__
    if verbosity >= 1:
        print ""
        chA_errors = flash_cmds.ttt.read("CHAFiberErrors")
        chB_errors = flash_cmds.ttt.read("CHBFiberErrors")
        eth_frame = flash_cmds.ttt.read("EthFrameErrors")
        
        print "Channel A Fiber Total Errors: %d" % ((chA_errors & 0x0000ffff) + ((chA_errors & 0xffff0000) >> 16))
        if verbosity >= 2:
            print "Channel A Fiber Single-Bit Errors: %d" % ((chA_errors & 0x0000ffff))
            print "Channel A Fiber Multi-Bit Errors: %d" % ((chA_errors & 0xffff0000) >> 16)
            
        print ""
        print "Channel B Fiber Total Errors: %d" % ((chB_errors & 0x0000ffff) + ((chB_errors & 0xffff0000) >> 16))
        if verbosity >= 2:
            print "Channel B Fiber Single-Bit Errors: %d" % ((chB_errors & 0x0000ffff))
            print "Channel B Fiber Multi-Bit Errors: %d" % ((chB_errors & 0xffff0000) >> 16)

        print ""
        print "Ethernet Frame Errors: %d" % eth_frame

status = 0
done_once = 0
raw_version = flash_cmds.ttt.read("VERSION")
__VERSION__ = "Firmware Version: %4d-%02d-%02d" % ((raw_version & 0xfff00000) >> 20, (raw_version & 0x000f0000) >> 16, (raw_version & 0x0000ff00) >> 8)
__SERIAL_NO__ = "Serial Number: %d" % (raw_version & 0xff)

while status == 0: #The loop continues forever. Program quits at user's command

    #Set add_offset to its default value
    add_offset = int(flash_cmds.flashAddOffset)

    #Print the menu the first time only
    if done_once == 0:
        print
        print_status(0)
        print_menu()

    done_once = 1

    #Read the user input
    badinput = True
    while badinput==True:
        input = raw_input("\nEnter flash action, 'h' for menu: ")
        if len(input) == 0:
            continue
        toks = input.split()
        command = toks[0].upper()
        #Quit if the user wants to
        if (command == "Q" or command == ".Q" or command == "QUIT"):
            sys.exit(0)
        #Reprint the menu if the user wants it
        elif (command == "H"):
            print_menu()
            continue
        #Else, take user flash command
        elif (command == "Z" or command == "V"
              or command == "P" or command == "L"
              or command == "RF" or command == "RS"
              or command == "WS" or command == "ST"
              or command == "BC"):
            badinput = False
        else:
            print "Invalid entry"

    #Make sure we can read from the chip
    try:
        test0 = flash_cmds.ttt.read("VERSION")
    except ChipsException:
        print "\nCannot read chip. Chip may not be configured."
        continue

    #Read flash pages
    if command == "RF":
        if len(toks) < 2 or len(toks) > 3:
            print "Invalid argument for command 'RF'"
            print "Must enter starting address and optionally the ",
            print "number of pages to be read for the flash page read\n"
            continue
        elif len(toks) == 2:
            try:
                add_offset = int(toks[1], 0)
            except ValueError:
                print "Provided address needs to be a number"
                continue
            num_pages = 1
        elif len(toks) == 3:
            try: 
                add_offset = int(toks[1], 0)
                num_pages = int(toks[2], 0)
            except ValueError:
                print "Provided address or no_pgs needs to be a number"
                continue
        flash_lib.printFlashPages(num_pages, add_offset)
        continue

    #Read spartan address
    if command == "RS":
        addr = 0
        if len(toks) < 2 or len(toks) > 3:
            print "Invalid argument for command 'RS'"
            print "Must enter starting address and optionally the ",
            print "number of addresses to be read from the spartan chip\n"
            continue
        elif len(toks) == 2:
            if is_int(toks[1]):
                read_addr = "VERSION"
                add_offset = int(toks[1], 0)
                addr = add_offset
                count = 1
            else:
                read_addr = toks[1]
                add_offset = 0
                count = 1
        elif len(toks) == 3:
            if is_int(toks[1]) and is_int(toks[2]):
                read_addr = "VERSION"                
                add_offset = int(toks[1], 0)
                addr = add_offset
                count = int(toks[2], 0)
                print "Address    Values"
            elif not is_int(toks[1]) and is_int(toks[2]):
                read_addr = toks[1]
                add_offset = 0
                addr = add_offset
                count = int(toks[2], 0)
                print "AddOff     Values"
            elif not is_int(toks[2]):
                print "Number of addresses to read must be an integer"
                continue
        for i in xrange(count):
            try:
                output = flash_cmds.ttt.read(read_addr, (add_offset+i))
                addr = get_address_from_name(read_addr)
            except ChipsException:
                print "Cannot read provided register using address table"
                continue
            if count <= 16:
                print "%08x: " % (addr+i),  "%08x" % output
            else:
                if not (i%8):
                    print "%08x: " % (addr+i),
                print "%08x" % output,
                if not ((i+1)%8):
                    print ""
        if (count%8):
            print ""

    if command == "ST":
        print ""
        if len(toks) == 2:
            if (is_int(toks[1])):
                print_status(int(toks[1]))
            else:
                print "<verbosity> must be an decimal integer value."
        elif len(toks) == 1:
            print_status()
        else:
            print "ST only takes one argument <verbosity>"

    if command == "BC":
        if len(toks) == 2:
            if is_int(toks[1]):
                bcast_cmd = int(toks[1], 0)
                flash_cmds.ttt.write("TTTBcastCmd", bcast_cmd, add_offset)
                flash_cmds.ttt.write("TTTActionReg", 0x02, add_offset)
                print "Sending Broadcast Command: 0x%x" % bcast_cmd
    #Write spartan address and value
    if command == "WS":
        if len(toks) != 3:
            print "Invalid argument for command 'WS'"
            print "Must enter address and value for the write"
            continue
        else:
            if is_int(toks[1]) and is_int(toks[2]):
                write_addr = "VERSION"
                add_offset = int(toks[1], 0)
                value = int(toks[2], 0)
            elif is_int(toks[2]):
                write_addr = toks[1]
                add_offset = 0
                value = int(toks[2], 0)
            else:
                print "Value to write must be an integer"
                continue
        try:
            flash_cmds.ttt.write(write_addr, value, add_offset)
        except ChipsException:
            print "Cannot write to provided register using address table"
            continue

    #Zero, Verify, or Program Flash
    if command == "Z" or command == "V" or command == "P":
        s = select_mcs.SelectMcsFile()
        if len(s) == 0:
            print ""
            continue
        #Make sure you want to write to flash
        if command == "Z":
            print "\nWarning!! You are about to zero the flash. Would you like to continue?"
            yo = raw_input("'y' for yes, any other key to abort: ")
        if command == "P":
            print "\nWARNING!! You are about to program the flash. Would you like to continue? "
            yo = raw_input("'y' for yes, any other key to abort: ")
        if command == "V":
            yo = 'Y'
        #Read the MCS file and do flash stuff
        if len(yo) != 0 and yo[0].upper() == 'Y':
            #parsing the mcs file
            d={}
            print "Reading MCS file.  May take many seconds..."
            flash_lib.fillDataDict(d,s)
            #get number of data bytes
            n=len(d)
            #get number of firmware pages
            npages=n/flash_cmds.flashPageSize
            #pad dictionary with zeros to fill the last page
            remainder=n%flash_cmds.flashPageSize
            if remainder !=0:
                npages +=1
                zerosneeded=flash_cmds.flashPageSize-remainder
                if(zerosneeded != 0):
                    for i in xrange(zerosneeded): d[n+i]=0
            print "\nfirmware file", s, "has", hex(n), "data bytes  (",npages," pages)"
            #zero the flash
            if command == "Z":
                flash_lib.zeroflash(npages,add_offset)
            #program the flash
            if command == "P":
                flash_lib.zeroflash(npages,add_offset)
                flash_lib.programflash(npages,add_offset,d)
                notGood = flash_lib.verifyflash(npages,add_offset,d)
                if not notGood:
                    print "\nTTT flash successfully programmed and verified"
                    print "Cycle the power to configure the FPGA"
            #read and verify the flash
            if command == "V":
                flash_lib.verifyflash(npages,add_offset,d)
        else:
            print "Flash action aborted"
        continue

    #Load section
    if command == "L":
        print "\nWarning!! You are about to reconfigure the FPGA from flash memory. Would you like to continue?"
        yol = raw_input("'y' for yes, any other key to abort: ")
        if (len(yol) != 0 and (yol[0] == 'y' or yol[0] == 'Y')):
            flash_cmds.ttt.write("CTRL", 0x100) #Reconfigure Spartan
            print "Please wait 5 seconds for the programming to finish"
            time.sleep(5)
            print "FPGA has been reconfigured from flash"
        continue
