#!/usr/bin/python
"""
TTT TTS Controller Readout Script
Christopher Woodall <cwoodall at bu.edu>
Boston University EDF <http://edf.bu.edu>
March 22, 2013
Rev. April 01, 2013
Charlie Hill <chill90 at bu.edu>

Version 1

Setup TTS_control fabric on the TTT board over IPBus.

@requires pychips
"""
VERSION=1
print "TTT TTS Controller Readout Script v%d" % VERSION
print "Boston University EDF <http://edf.bu.edu> "
print "Chris Woodall <cwoodall at bu.edu>"
print ""

import ttt as ttt
import sys
import os
from optparse import OptionParser

def writeAR(b, value):
    AR_COMMANDS = {"read_fifo": 2, "latch_count": 4, "request_l1a": 1, "reset": 128}
    v = AR_COMMANDS[value]
    # Clear Action Register
    b.write("TTSActionReg", 0)
    # Write Action Register
    b.write("TTSActionReg", v)
    # Clear Action Register
    b.write("TTSActionReg", 0)    

def decode(v):
    if v == 0x1:
        return "OVF"
    elif v == 0x2:
        return "SYN"
    elif v == 0x4:
        return "BSY"
    elif v == 0x8:
        return "RDY"
    else:
        return "ERR"
    
# Setup OptionParser
parser = OptionParser()
parser.add_option("-t", dest="l1atest", help="test",
                  default=0, metavar="<test>", type=int)

(options, args) = parser.parse_args()

print("L1ATest       -t %d" % options.l1atest)
print("")
#/* read counters */
#/* select counters */
ttt.write("TTSFifoByte",14)
for i in range(4):
    ttt.write("TTSCounterByte",i)
    v = ttt.read("TTSDataReg")
    print("Counter %d = %d" % (i,v))

# Disable any ongoing repeats
##ttt.write("TTSActionReg", 1<<7);
print
d = 0
ts0 = 0
n = 0
while (d & 0x1) == 0:
    d = (ttt.read("TTSStatusReg"))
    fifo = 0
    for i in reversed(range(8)):
        ttt.write("TTSFifoByte", i);
        v = ttt.read("TTSDataReg")
        fifo |= v << (8*i);
    ts = fifo >> 4;
    dts = ts - ts0;
    dt = dts * 25e-9;
    v = fifo & 0xf
    print("%3d: -> %x (%s)  (%20d BX, dt = %7.3g sec)\n" % (n, v, decode(v), dts, dt))
    n += 1
    writeAR(ttt, "read_fifo")
    ts0 = ts;
