
#Wrapper module for ChipsBusUdp to give us a little more control
#over the read and write functionality of the TTT

from PyChipsUser import *
import os
import sys
from time import sleep

#IMPORTANT VARIABLES FOR USING THE TTT FLASH!
FLASH_ADD_OFFSET = 0x000000
FLASH_SECTOR_SIZE = (1<<16)       # sectors are 256k bytes
FLASH_PAGE_SIZE = 256             # each page has 256 bytes
FLASH_PAGES_PER_SECTOR = FLASH_SECTOR_SIZE/FLASH_PAGE_SIZE

#IMPORTANT VARIABLES FOR ACCESSING THE TTT!
IPBUS_PORT = 50001
IPADDR_BASE = "192.168.2"
IPADDR_START = 0x20

#Obtain IP Address
try:
    serialNumber = int(os.environ["TTT_SERIAL_NUMBER"])
except KeyError:
    print "$TTT_SERIAL_NUMBER not set. Run the 'enviorn.sh' script before running python scripts"
    print "Quitting..."
    sys.exit(1)
ttt_ip = IPADDR_BASE+"."+str(IPADDR_START+serialNumber)
ttt_port = IPBUS_PORT

#Obtain address table path
ttt_ad = str(os.environ["TTT_HOME"])+"/map/addrTable.txt"

#Print ID information
print "IP Address: " + ttt_ip
print "Address Table: " + ttt_ad

#Create PyChips object
pyAddTbl = AddressTable(ttt_ad)
ttt = ChipsBusUdp(pyAddTbl, ttt_ip, int(ttt_port))

#Check to make sure we can talk to the board at the given ip address
err = 0
try:
    ttt.read("FirmwareVersionID")
except ChipsException:
    err = 1
    pass

if err == 1:
    try:
        ttt.read("FirmwareVersionID")
    except ChipsException:
        print "Trouble talking to board at provided IP Address..."
        print "Quitting..."
        sys.exit(1)
    

#Number of times to retry after a failed write or read
numRetrys = 2 #Should be set to 0 if firmware working correctly
delay = .1  #In seconds


def FatalError(msg):
    print "FatalError(): " + msg
    print "Quitting..."
    sys.exit(1)
    

def write(reg, val, off=0):
    i = 0
    while i <= numRetrys:
        try:
            ttt.write(reg, val, off)
            return
        except ChipsException:
            i += 1
            sleep(delay)
            continue
    FatalError("Trouble writing to '"+reg+"'")
    

def writeBlock(reg, vec):
    i = 0
    while i <= numRetrys:
        try:
            ttt.blockWrite(reg, vec)
            return
        except ChipsException:
            i += 1
            sleep(delay)
            continue
    FatalError("Trouble block writing to "+reg+"'")
    

def read(reg, off=0):
    i = 0
    while i <= numRetrys:
        try:
            val = ttt.read(reg, off)
            return val
        except:
            i += 1
            sleep(delay)
            continue
    FatalError("Trouble writing to '"+reg+"'")
    

def readBlock(reg, num):
    i = 0
    retVec = []
    while i <= numRetrys:
        try:
            retVec = ttt.blockRead(reg, num)
            return retVec
        except ChipsException:
            i += 1
            sleep(delay)
            continue
    FatalError("Trouble block reading from '"+reg+"'")
